#pragma once

#include "windows.h"
#include "winhttp.h"
#include "basic/akl_string.h"
#include "CCaseManagment.h"

class CProgressCtrl;
class CStatic;

/// Class is a wrapper for network functions.
class CaseServer
{
public:
    /// Stops current action (download / upload).
    static void StopAction();

public:
    /// Constructor.
    /// \param caseApiUrl Url of case server.
    CaseServer(akl_string caseApiUrl);
    /// Destructor.
    ~CaseServer();

    /// Makes CaseServer class to use proxy server.
    /// \param proxyUrl Url of proxy server.
    /// \param proxyUser User name for proxy server.
    /// \param proxyPass Password for proxy server.
    void UseProxy(akl_string proxyUrl, akl_string proxyUser, akl_string proxyPass);
    /// Turn on/off using of SSL.
    /// \param isUseSsl If true turns using of SSL on; otherwise off.
    void UseSsl(bool isUseSsl);

    // Case server operations.
    /// Authenticates user on server.
    /// \return True if authentication  is successful; otherwise false.
    bool Authenticate(akl_string userName, akl_string password);
    /// Gets xml case list from server.
    /// \param toBeMarked Flag identifying which case list to download.
    /// \return String containing xml case data.
    akl_string DownloadCaseList(bool toBeMarked);
    /// Downloads case from server.
    /// \param aCase Case to be downloaded.
    /// \param downloadBar Progress bar to be updated during download process.
    /// \param lableStatus Status lable to be updated during download process.
    /// \return True if downloading complete successfully; otherwise false.
    bool DownloadCase(struct _casem& aCase, CProgressCtrl* downloadBar, CStatic* labelStatus);
    /// Uploads case from server.
    /// \param aCase Case to be uploaded.
    /// \return True if uploading complete successfully; otherwise false.
    bool UploadCase(struct _casem& aCase);
    /// Uploads info about articulator.
    /// \param caseNumber ID of case.
    /// \param articulatorType String containing articulator type.
    /// \return True if articulator is uploaded successuflly; ohterwise false.
    bool UploadArticulator(akl_string caseNumber, akl_string articulatorType);
    /// Gets value to indicate if CaseServer is in error state.
    /// \return True if CaseServer is in error state; otherwise false.
    bool IsError();
    /// Gets value to indicate if CaseServer is in error state.
    /// \return True if CaseServer is in error state; otherwise false.
    bool IsConnected();
    /// Returns error description. Valid if IsError() method return true.
    /// \return String containing error description.
    akl_string GetErrorString();
    /// Returns error code
    /// \return Returns error code
    int GetErrorCode();
    /// Shows whether user was authentificated or not.
    /// \return True if user was authentificated; otherwise false.
    bool IsAuthentificated();
    /// Updates download UI controls.
    /// \param downloadBar Progress bar to be updated during download process.
    /// \param lableStatus Status lable to be updated during download process.
    void UpdateDownloadUI(CProgressCtrl* downloadBar, CStatic* labelStatus, int readBytes);
    
private:
    // Enumerates supported request types.
    enum RequestType { eGet = 0, ePost, ePut };
    /// Creates request of specified type.
    /// \param requestUrl Resource on server.
    /// \param type Type of request to create. By default "GET" is used.
    /// \return True if request was created; otherwise false.
    bool CreateRequest(akl_string requestUrl, RequestType type = eGet);
    /// Get response and return status code.
    /// \return Status code obtained from respose. 
    /// On errors -1 is returned.
    DWORD GetResponse();
    /// Reads response data to buffer.
    /// \param[out] Reference to pointer that is initialized to point to
    /// buffer with data from server. 
    /// \return Number of bytes that was read. On errors -1 is returned.
    int  ReadResponse(char** ppBuffer);
    /// Opens connection to server.
    /// \return True if connection is opened successfully; otherwise false.
    bool OpenConnection();
    /// Set credentials (server and proxy) as appropiate.
    /// \return True if operation is successful; otherwise false.
    bool SetCredentials();
    /// Cloases connection and reset to unconnected state.
    void CloseConnection();
    // Supported headers with number values.
    enum HeaderNumberValue { eStatusCode=0, eContentLength };
    // Supported headers with number values.
    DWORD GetHeaderNumberValue(HeaderNumberValue header);

    /// Returns user agent name.
    /// \return String contained user agent 
    akl_string GetVammAgent();
    /// Fixes specified URL.
    /// Fix url so that "https://test.com/something/somethingelse" becomes "test.com"
    /// \param url URL to be fixed.
    /// \return String with a fixed url.
    akl_string FixUrl(akl_string url);
    /// Fixes specified resource URL.
    /// Fix url so that "https://test.com/something/somethingelse" 
    /// becomes "/something/somethingelse"
    /// \param url URL to be fixed.
    /// \return String with a fixed url.
    akl_string FixResourceUrl(akl_string url);
    /// Sets specified error string and code.
    /// \param errorString String containing error description.
    /// \param errorCode Error code. By default used -1.
    void SetError(akl_string errorString, int errorCode = -1);
    /// Sets HTTP error string for specified error code.
    /// \param errorCode HTTP status code retiurnd by response.
    void SetHttpError(int errorCode);
   
private:
    /// Case api url
    akl_string m_caseApiUrl;
    /// SSL control flag.
    bool m_isUseSsl;
    /// Proxy control.
    bool m_isUseProxy;
    /// Proxy url.
    akl_string m_proxyUrl;
    akl_string m_proxyUser;
    akl_string m_proxyPass;
    /// Connection control.
    bool m_isConnected;    
    HINTERNET m_hSession;
    HINTERNET m_hConnection;
    HINTERNET m_hRequest;
    /// Error managment.
    bool       m_isError;
    int        m_errorCode;
    akl_string m_errorString;
    /// Content length in kb.
    int m_contentLength;
    /// in bytes
    DWORD m_totalDownloaded;
private:
    /// Server credentials.
    static akl_string m_userName;
    static akl_string m_password;
    /// Flag to show that download/upload must be stopped.
    static bool m_stopAction;
};

inline void CaseServer::StopAction()
{
    m_stopAction = true;
}

inline void CaseServer::UseSsl(bool isUseSsl)
{
    m_isUseSsl = isUseSsl;
}

inline bool CaseServer::IsConnected()
{
    return m_isConnected;
}

inline bool CaseServer::IsError()
{
    return m_isError;
}

inline akl_string CaseServer::GetErrorString()
{
    return m_errorString;
}

inline int CaseServer::GetErrorCode()
{
    return m_errorCode;
}

inline bool CaseServer::IsAuthentificated()
{
    return (CaseServer::m_userName.length() && CaseServer::m_password.length());
}